package_name = 'aldryn_search'
name = 'aldryn-search'
author = 'Benjamin Wohlwend'
author_email = 'piquadrat@gmail.com'
description = "An extension to django CMS to provide multilingual Haystack indexes"
version = __import__(package_name).__version__
project_url = 'http://github.com/aldryn/%s' % name
license = 'BSD'
